# LAN/COM/USB-CDC/USB-TMC/GP-IB Access Sample                Ver1.00
# 本サンプルはWindowsでのみ動作確認となります。
# NI-VISAのフルインストール版が必要です。
# 対象電源はPSW/PFR/PSU/PPX/ASRシリーズです。
# PythonからVISAの機器と通信する場合はpyVISAモジュールを利用します。
# GUI表示はTkinterを使用しています。
# デバイスを検索し一覧表示から選択し通信を行います。
# Socketデバイスの場合はNI-MAXに登録してから利用してください。

# coding:utf-8
import pyvisa as visa   # VISAモジュール
import tkinter as tk    # tkモジュール
import time
from time import sleep
from tkinter import ttk

rm = visa.ResourceManager()                 #rmとinstrをグローバルとして定義
instr = None

def Find_clicked():
    try:
# デバイスを検索し一覧表示します
        cbDevList['values']=['']

        instruments = rm.list_resources('?*')
        if len(instruments) > 0:
            cbDevList['values']=instruments
            cbDevList.current(0)              # Error処理
            btn3['state'] = tk.NORMAL 
    except:
        print('Device Error')

def Exit_clicked():                           #Exitボタン処理

    rm.close()          # デバイスとの通信を終了します。
    root.destroy()

def Open_clicked():
    global instr 
    visaAddr = cbDevList.get()
    if( visaAddr == '' ):
        print('No Device!')
    else:
        try:
            instr = rm.open_resource(visaAddr)
            if instr.resource_name.startswith('ASRL'):
                instr.read_termination = '\n'
                instr.write_termination = '\n'
                instr.baud_rate = int(txtOption.get())

            if instr.resource_name.endswith('SOCKET'):
                instr.read_termination = '\n'
                instr.write_termination = '\n'
        
            btn2['state'] = tk.DISABLED     # OPEN/EXIT禁止､CLOSE/Send/Recive有効
            btn3['state'] = tk.DISABLED
            btn4['state'] = tk.NORMAL
            btn5['state'] = tk.NORMAL
            btn6['state'] = tk.NORMAL
            btn7['state'] = tk.NORMAL
            btn8['state'] = tk.NORMAL
            btn9['state'] = tk.NORMAL

            instr.write('*idn?')
            txtRecive.delete('1.0',tk.END)
            txtRecive.insert (tk.END,' Device Opened!\n'+instr.read())

# Error処理
        except:

            txtRecive.delete('1.0',tk.END)
            txtRecive.insert (tk.END,' Device Open Error!')

def cmb_Sel(event):                              #コンボボックス選択処理
    txtOption.delete(0,tk.END)
    cbuff = cbDevList.get()
    if('ASRL' in cbuff):
        txtOption['state'] = tk.NORMAL
        txtOption.insert(tk.END,'9600')
    else:
        txtOption['state'] = tk.DISABLED


def Close_clicked():                             #Closeボタン処理
    global instr                                 #Globalのinstr変数を利用

    instr.write('syst:loc')

    btn2['state'] = tk.NORMAL     # OPEN/EXIT有効､CLOSE/Send/Recive禁止
    btn3['state'] = tk.NORMAL
    btn4['state'] = tk.DISABLED
    btn5['state'] = tk.DISABLED
    btn6['state'] = tk.DISABLED
    btn7['state'] = tk.DISABLED
    btn8['state'] = tk.DISABLED
    btn9['state'] = tk.DISABLED
    btn10['state'] = tk.DISABLED

    instr.close()                                #VISAデバイスをクロース

def Send_clicked():                              #送信ボタン処理
    global instr                                 #Globalのinstr変数を利用

    cbuff = txtSend.get()                        #テキストボックスから文字列を取得
    instr.write(cbuff)                           #文字列をVISAで送信
    txtRecive.delete('1.0',tk.END)               #テキストを消去
    if ('?' in cbuff) :                          #送信文字列に？がある場合
        txtRecive.insert (tk.END,instr.read())   #  文字列をVISA受信し表示

def Recive_clicked():                            #受信ボタン処理
    global instr                                 #Globalのinstr変数を利用

    txtRecive.delete('1.0',tk.END)               #テキストを消去
    txtRecive.insert (tk.END,instr.read())       #文字列をVISA受信し表示

def OUTON_clicked():                               #送信ボタン処理
    global instr                                 #Globalのinstr変数を利用

    instr.write(':OUTP ON')                      #文字列をVISAで送信

def OUTOFF_clicked():                            #送信ボタン処理
    global instr                                 #Globalのinstr変数を利用

    instr.write(':OUTP OFF')                     #文字列をVISAで送信

def LOOP_clicked():
    global loopFlg
    loopFlg = 1 
    btn4['state'] = tk.DISABLED
    btn5['state'] = tk.DISABLED
    btn6['state'] = tk.DISABLED
    btn9['state'] = tk.DISABLED
    btn10['state'] = tk.NORMAL

    txtRecive.delete('1.0',tk.END)

    interval_work()

def STOP_clicked():
    global loopFlg
    loopFlg = 0
    btn4['state'] = tk.NORMAL
    btn5['state'] = tk.NORMAL
    btn6['state'] = tk.NORMAL
    btn9['state'] = tk.NORMAL
    btn10['state'] = tk.DISABLED


def interval_work():                            # インターバルで行う処理を記述
    global loopFlg
    global instr
    
    instr.write(':MEAS:VOLT?;:MEAS:CURR?\n')
    sleep(0.1)
    txtRcv = instr.read() 
    txtRecive.insert(tk.END,time.ctime() + '   ,' + txtRcv.replace('\n','') +'\n' )
    txtRecive.see('end')
    if (loopFlg == 1):
        root.after(1000, interval_work)              # インターバルの時間(ms)を指定
        


root = tk.Tk()
root.geometry('540x200')
root.title('pyVisa Sample for POWER')

label1 = tk.Label(root, text = 'Device List  :  ')
label1.grid(row=0, column=0)
label2 = tk.Label(root,text = 'Serial Baud :' )
label2.grid(row=1, column=0 )

cbDevList= ttk.Combobox(root , width=50 )
cbDevList.grid(row=0 , column=1)
cbDevList.bind('<<ComboboxSelected>>',cmb_Sel)

txtOption = ttk.Entry(root , width=50 ,   state =tk.DISABLED )
txtOption.grid(row=1 , column=1)
txtSend = ttk.Entry(root ,  width=50 )
txtSend.grid(row=2 , column=1)
txtSend.insert(tk.END,'*idn?')

txtRecive = tk.Text(root , width=44, height=5 )
txtRecive.grid(row=4 , column=1)

btn1 = tk.Button(master=root, text='Find'   , command=Find_clicked,width= 7)
btn1.grid(row = 0 , column=2)
btn2 = tk.Button(master=root, text='Exit'   , command=Exit_clicked,width= 7)
btn2.grid(row = 0 , column=3)
btn3 = tk.Button(master=root, text='Open'   , command=Open_clicked,   state =tk.DISABLED ,width= 7)
btn3.grid(row = 1 , column=2)
btn4 = tk.Button(master=root, text='Close'  , command=Close_clicked,  state =tk.DISABLED ,width= 7)
btn4.grid(row = 1 , column=3)
btn5 = tk.Button(master=root, text='Send'   , command=Send_clicked,   state =tk.DISABLED ,width= 7)
btn5.grid(row = 2 , column=0)
btn6 = tk.Button(master=root, text='Recive' , command=Recive_clicked, state =tk.DISABLED ,width= 7)
btn6.grid(row = 3 , column=0)
btn7 = tk.Button(master=root, text='OUT OFF', command=OUTOFF_clicked, state =tk.DISABLED ,width= 7)
btn7.grid(row = 3 , column=2)
btn8 = tk.Button(master=root, text='OUT ON ', command=OUTON_clicked,  state =tk.DISABLED ,width= 7)
btn8.grid(row = 3 , column=3)
btn9 = tk.Button(master=root, text='LOOP'   , command=LOOP_clicked,   state =tk.DISABLED ,width= 7)
btn9.grid(row = 4 , column=2)
btn10 = tk.Button(master=root, text='Stop'  , command=STOP_clicked,  state =tk.DISABLED ,width= 7)
btn10.grid(row = 4 , column=3)
        

root.mainloop()

